@@ -2,9 +2,8 @@
 #
 # Program to take a set of header files and generate DLL export definitions
 
-# Export Objective-C classes for Cocoa integration support
+# print objective-c exports
 print "\t.objc_class_name_SDL_QuartzWindow\n";
-print "\t.objc_class_name_SDL_QuartzWindowView\n";
 print "\t.objc_class_name_SDL_QuartzWindowDelegate\n";
 
 while ( ($file = shift(@ARGV)) ) {
