@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_blit_A.c,v 1.5 2003/08/22 05:51:19 slouken Exp $";
+ "@(#) $Id: SDL_blit_A.c,v 1.8 2004/01/04 16:49:21 slouken Exp $";
 #endif
 
 #include <stdio.h>
@@ -32,16 +32,9 @@ static char rcsid =
 #include "SDL_blit.h"
 
 #if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
-#include "mmx.h"
 /* Function to check the CPU flags */
-#define MMX_CPU		0x800000
-#define TDNOW_CPU	0x80000000
-#define CPU_Flags()	Hermes_X86_CPU()
-#define X86_ASSEMBLER
-#define HermesConverterInterface	void
-#define HermesClearInterface		void
-#define STACKCALL
-#include "HeadX86.h"
+#include "SDL_cpuinfo.h"
+#include "mmx.h"
 #endif
 
 /* Functions to perform alpha blended blitting */
@@ -220,11 +213,11 @@ static void BlitRGBtoRGBSurfaceAlpha128MMX(SDL_BlitInfo *info)
 	int dstskip = info->d_skip >> 2;
         Uint8 load[8];
   
-        *(Uint64 *)load = 0x00fefefe00fefefe;/* alpha128 mask */
+        *(Uint64 *)load = 0x00fefefe00fefefeULL;/* alpha128 mask */
         movq_m2r(*load, mm4); /* alpha128 mask -> mm4 */
-        *(Uint64 *)load = 0x0001010100010101;/* !alpha128 mask */
+        *(Uint64 *)load = 0x0001010100010101ULL;/* !alpha128 mask */
         movq_m2r(*load, mm3); /* !alpha128 mask -> mm3 */
-        *(Uint64 *)load = 0xFF000000FF000000;/* dst alpha mask */
+        *(Uint64 *)load = 0xFF000000FF000000ULL;/* dst alpha mask */
         movq_m2r(*load, mm7); /* dst alpha mask -> mm7 */
 	while(height--) {
             DUFFS_LOOP_DOUBLE2(
@@ -276,10 +269,10 @@ static void BlitRGBtoRGBSurfaceAlphaMMX(SDL_BlitInfo *info)
     					alpha, alpha, alpha, alpha};
 					
                 movq_m2r(*load, mm4); /* alpha -> mm4 */
-		*(Uint64 *)load = 0x00FF00FF00FF00FF;
+		*(Uint64 *)load = 0x00FF00FF00FF00FFULL;
                 movq_m2r(*load, mm3); /* mask -> mm3 */
 		pand_r2r(mm3, mm4); /* mm4 & mask -> 0A0A0A0A -> mm4 */
-		*(Uint64 *)load = 0xFF000000FF000000;/* dst alpha mask */
+		*(Uint64 *)load = 0xFF000000FF000000ULL;/* dst alpha mask */
 		movq_m2r(*load, mm7); /* dst alpha mask -> mm7 */
 		
 		while(height--) {
@@ -360,13 +353,13 @@ static void BlitRGBtoRGBPixelAlphaMMX(SDL_BlitInfo *info)
         Uint32 alpha = 0;
         Uint8 load[8];
 	                
-	*(Uint64 *)load = 0x00FF00FF00FF00FF;
+	*(Uint64 *)load = 0x00FF00FF00FF00FFULL;
         movq_m2r(*load, mm3); /* mask -> mm2 */
-	*(Uint64 *)load = 0x00FF000000000000;
+	*(Uint64 *)load = 0x00FF000000000000ULL;
         movq_m2r(*load, mm7); /* dst alpha mask -> mm2 */
-        *(Uint64 *)load = 0x00FFFFFF00FFFFFF;
+        *(Uint64 *)load = 0x00FFFFFF00FFFFFFULL;
         movq_m2r(*load, mm0); /* alpha 255 mask -> mm0 */
-        *(Uint64 *)load = 0xFF000000FF000000;
+        *(Uint64 *)load = 0xFF000000FF000000ULL;
         movq_m2r(*load, mm6); /* alpha 255 !mask -> mm6 */
 	while(height--) {
 	    DUFFS_LOOP4({
@@ -792,11 +785,11 @@ static void Blit565to565SurfaceAlphaMMX(SDL_BlitInfo *info)
                 punpcklwd_r2r(mm0, mm0); /* 0A0A0A0A -> mm0 */
 	  
  	        /* Setup the 565 color channel masks */
-	        *(Uint64 *)load = 0xF800F800F800F800;
+	        *(Uint64 *)load = 0xF800F800F800F800ULL;
 		movq_m2r(*load, mm1); /* MASKRED -> mm1 */
-		*(Uint64 *)load = 0x07E007E007E007E0;
+		*(Uint64 *)load = 0x07E007E007E007E0ULL;
 		movq_m2r(*load, mm4); /* MASKGREEN -> mm4 */
-		*(Uint64 *)load = 0x001F001F001F001F;
+		*(Uint64 *)load = 0x001F001F001F001FULL;
 		movq_m2r(*load, mm7); /* MASKBLUE -> mm7 */
 		while(height--) {
                         DUFFS_LOOP_QUATRO2(
@@ -942,11 +935,11 @@ static void Blit555to555SurfaceAlphaMMX(SDL_BlitInfo *info)
                 punpcklwd_r2r(mm0, mm0); /* 0A0A0A0A -> mm0 */
 	  
  	        /* Setup the 555 color channel masks */
-	        *(Uint64 *)load = 0x7C007C007C007C00;
+	        *(Uint64 *)load = 0x7C007C007C007C00ULL;
 		movq_m2r(*load, mm1); /* MASKRED -> mm1 */
-		*(Uint64 *)load = 0x03E003E003E003E0;
+		*(Uint64 *)load = 0x03E003E003E003E0ULL;
 		movq_m2r(*load, mm4); /* MASKGREEN -> mm4 */
-		*(Uint64 *)load = 0x001F001F001F001F;
+		*(Uint64 *)load = 0x001F001F001F001FULL;
 		movq_m2r(*load, mm7); /* MASKBLUE -> mm7 */
 		while(height--) {
                         DUFFS_LOOP_QUATRO2(
@@ -1387,7 +1380,7 @@ SDL_loblit SDL_CalculateAlphaBlit(SDL_Surface *surface, int blit_index)
 		    if(df->Gmask == 0x7e0)
 		    {
 #if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
-		if((CPU_Flags()&MMX_CPU)!=0)
+		if(SDL_HasMMX())
 			return Blit565to565SurfaceAlphaMMX;
 		else
 #endif
@@ -1396,7 +1389,7 @@ SDL_loblit SDL_CalculateAlphaBlit(SDL_Surface *surface, int blit_index)
 		    else if(df->Gmask == 0x3e0)
 		    {
 #if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
-		if((CPU_Flags()&MMX_CPU)!=0)
+		if(SDL_HasMMX())
 			return Blit555to555SurfaceAlphaMMX;
 		else
 #endif
@@ -1413,7 +1406,7 @@ SDL_loblit SDL_CalculateAlphaBlit(SDL_Surface *surface, int blit_index)
 		   && sf->BytesPerPixel == 4)
 		{
 #if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
-		if((CPU_Flags()&MMX_CPU)!=0)
+		if(SDL_HasMMX())
 		    return BlitRGBtoRGBSurfaceAlphaMMX;
 		else
 #endif
@@ -1453,12 +1446,10 @@ SDL_loblit SDL_CalculateAlphaBlit(SDL_Surface *surface, int blit_index)
 	       && sf->BytesPerPixel == 4)
 	    {
 #if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
-		Uint32 f;
-		f=CPU_Flags();
-		if((f&(TDNOW_CPU|MMX_CPU))==(TDNOW_CPU|MMX_CPU))
+		if(SDL_Has3DNow())
 		    return BlitRGBtoRGBPixelAlphaMMX3DNOW;
 		else
-		if((f&MMX_CPU)!=0)
+		if(SDL_HasMMX())
 		    return BlitRGBtoRGBPixelAlphaMMX;
 		else
 #endif
