@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_DirectFB_video.h,v 1.6 2002/08/31 04:13:28 slouken Exp $";
+ "@(#) $Id: SDL_DirectFB_video.h,v 1.8 2004/01/04 16:49:24 slouken Exp $";
 #endif
 
 #ifndef _SDL_DirectFB_video_h
@@ -47,6 +47,16 @@ struct SDL_PrivateVideoData
 
   int nummodes;
   SDL_Rect **modelist;
+
+  /* MGA CRTC2 support */
+  int enable_mga_crtc2;
+  int mga_crtc2_stretch;
+  float mga_crtc2_stretch_overscan;
+  IDirectFBDisplayLayer *c2layer;
+  IDirectFBSurface *c2frame;
+  DFBRectangle c2ssize;	/* Real screen size */
+  DFBRectangle c2dsize;	/* Stretched screen size */
+  DFBRectangle c2framesize;    /* CRTC2 screen size */
 };
 
 #define HIDDEN (this->hidden)
