@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_ph_video.c,v 1.19 2003/08/30 17:07:59 slouken Exp $";
+ "@(#) $Id: SDL_ph_video.c,v 1.24 2004/02/17 16:19:29 slouken Exp $";
 #endif
 
 #include <stdlib.h>
@@ -64,19 +64,23 @@ static void* ph_GL_GetProcAddress(_THIS, const char* proc);
 static int   ph_GL_MakeCurrent(_THIS);
 #endif /* HAVE_OPENGL */
 
+static int phstatus=-1;
+
 static int ph_Available(void)
 {
-    int phstat=-1;
-
-    phstat=PtInit(0);
-    if (phstat==0)
+    if (phstatus!=0)
     {
-       return 1;
-    }
-    else
-    {
-       return 0;
+        phstatus=PtInit(NULL);
+        if (phstatus==0)
+        {
+           return 1;
+        }
+        else
+        {
+           return 0;
+        }
     }
+    return 1;
 }
 
 static SDL_VideoDevice *ph_CreateDevice(int devindex)
@@ -112,10 +116,10 @@ static SDL_VideoDevice *ph_CreateDevice(int devindex)
     device->UpdateRects = NULL;         /* set up in ph_SetupUpdateFunction */
     device->VideoQuit = ph_VideoQuit;
     device->AllocHWSurface = ph_AllocHWSurface;
-    device->CheckHWBlit = NULL;
-    device->FillHWRect = NULL;
-    device->SetHWColorKey = NULL;
-    device->SetHWAlpha = NULL;
+    device->CheckHWBlit = ph_CheckHWBlit;
+    device->FillHWRect = ph_FillHWRect;
+    device->SetHWColorKey = ph_SetHWColorKey;
+    device->SetHWAlpha = ph_SetHWAlpha;
     device->LockHWSurface = ph_LockHWSurface;
     device->UnlockHWSurface = ph_UnlockHWSurface;
     device->FlipHWSurface = ph_FlipHWSurface;
@@ -182,7 +186,7 @@ static PtWidget_t *ph_CreateWindow(_THIS)
 {
     PtWidget_t *widget;
     
-    widget = PtCreateWidget(PtWindow, NULL, 0, 0);
+    widget = PtCreateWidget(PtWindow, NULL, 0, NULL);
 
     return widget;
 }
@@ -191,20 +195,28 @@ static int ph_SetupWindow(_THIS, int w, int h, int flags)
 {
     PtArg_t     args[32];
     PhPoint_t   pos = {0, 0};
+    PhDim_t*    olddim;
     PhDim_t     dim = {w, h};
+    PhRect_t    desktopextent;
     int         nargs = 0;
     const char* windowpos;
     const char* iscentered;
     int         x, y;
 
-    PtSetArg(&args[nargs++], Pt_ARG_DIM, &dim, 0);
+    /* check if window size has been changed by Window Manager */
+    PtGetResource(window, Pt_ARG_DIM, &olddim, 0);
+    if ((olddim->w!=w) || (olddim->h!=h))
+    {
+       PtSetArg(&args[nargs++], Pt_ARG_DIM, &dim, 0);
+    }
 
     if ((flags & SDL_RESIZABLE) == SDL_RESIZABLE)
     {
-        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_MANAGED_FLAGS, Pt_FALSE, Ph_WM_RESIZE | Ph_WM_CLOSE);
-        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_MANAGED_FLAGS, Pt_TRUE, Ph_WM_MAX | Ph_WM_RESTORE);
-        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_NOTIFY_FLAGS, Pt_TRUE, Ph_WM_RESIZE | Ph_WM_MOVE | Ph_WM_MAX | Ph_WM_RESTORE | Ph_WM_CLOSE);
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_MANAGED_FLAGS, Pt_FALSE, Ph_WM_CLOSE);
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_MANAGED_FLAGS, Pt_TRUE, Ph_WM_MAX | Ph_WM_RESTORE | Ph_WM_RESIZE);
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_NOTIFY_FLAGS, Pt_TRUE, Ph_WM_RESIZE | Ph_WM_MOVE | Ph_WM_CLOSE | Ph_WM_MAX | Ph_WM_RESTORE);
         PtSetArg(&args[nargs++], Pt_ARG_WINDOW_RENDER_FLAGS, Pt_TRUE, Ph_WM_RENDER_RESIZE | Ph_WM_RENDER_MAX | Ph_WM_RENDER_COLLAPSE | Ph_WM_RENDER_RETURN);
+        PtSetArg(&args[nargs++], Pt_ARG_RESIZE_FLAGS, Pt_TRUE, Pt_RESIZE_XY_AS_REQUIRED);
     }
     else
     {
@@ -212,6 +224,7 @@ static int ph_SetupWindow(_THIS, int w, int h, int flags)
         PtSetArg(&args[nargs++], Pt_ARG_WINDOW_NOTIFY_FLAGS, Pt_FALSE, Ph_WM_RESIZE | Ph_WM_MAX | Ph_WM_RESTORE);
         PtSetArg(&args[nargs++], Pt_ARG_WINDOW_NOTIFY_FLAGS, Pt_TRUE, Ph_WM_MOVE | Ph_WM_CLOSE);
         PtSetArg(&args[nargs++], Pt_ARG_WINDOW_RENDER_FLAGS, Pt_FALSE, Ph_WM_RENDER_RESIZE | Ph_WM_RENDER_MAX | Ph_WM_RENDER_COLLAPSE | Ph_WM_RENDER_RETURN);
+        PtSetArg(&args[nargs++], Pt_ARG_RESIZE_FLAGS, Pt_FALSE, Pt_RESIZE_XY_AS_REQUIRED);
     }
 
     if (((flags & SDL_NOFRAME)==SDL_NOFRAME) || ((flags & SDL_FULLSCREEN)==SDL_FULLSCREEN))
@@ -236,45 +249,88 @@ static int ph_SetupWindow(_THIS, int w, int h, int flags)
     {
         PtSetArg(&args[nargs++], Pt_ARG_POS, &pos, 0);
         PtSetArg(&args[nargs++], Pt_ARG_BASIC_FLAGS, Pt_TRUE, Pt_BASIC_PREVENT_FILL);
-        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_MANAGED_FLAGS, Pt_TRUE, Ph_WM_FFRONT | Ph_WM_MAX);
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_MANAGED_FLAGS, Pt_TRUE, Ph_WM_FFRONT | Ph_WM_MAX | Ph_WM_TOFRONT | Ph_WM_CONSWITCH);
         PtSetArg(&args[nargs++], Pt_ARG_WINDOW_STATE, Pt_TRUE, Ph_WM_STATE_ISFRONT | Ph_WM_STATE_ISFOCUS | Ph_WM_STATE_ISALTKEY);
     }
     else
     {
-        windowpos = getenv("SDL_VIDEO_WINDOW_POS");
-	iscentered = getenv("SDL_VIDEO_CENTERED");
-
-        if ((iscentered) || ((windowpos) && (strcmp(windowpos, "center")==0)))
+        if ((flags & SDL_HWSURFACE) == SDL_HWSURFACE)
         {
-            pos.x = (desktop_mode.width - w)/2;
-            pos.y = (desktop_mode.height - h)/2;
-            PtSetArg(&args[nargs++], Pt_ARG_POS, &pos, 0);
-	}
+            PtSetArg(&args[nargs++], Pt_ARG_BASIC_FLAGS, Pt_TRUE, Pt_BASIC_PREVENT_FILL);
+        }
         else
         {
-            if (windowpos)
+            PtSetArg(&args[nargs++], Pt_ARG_FILL_COLOR, Pg_BLACK, 0);
+        }
+        if (!currently_maximized)
+        {
+            windowpos = getenv("SDL_VIDEO_WINDOW_POS");
+            iscentered = getenv("SDL_VIDEO_CENTERED");
+
+            if ((iscentered) || ((windowpos) && (strcmp(windowpos, "center")==0)))
             {
-                if (sscanf(windowpos, "%d,%d", &x, &y) == 2 )
+                PhWindowQueryVisible(Ph_QUERY_CONSOLE, 0, 0, &desktopextent);
+                if (desktop_mode.width>w)
                 {
-                    pos.x=x;
-                    pos.y=y;
-                    PtSetArg(&args[nargs++], Pt_ARG_POS, &pos, 0);
+                    pos.x = (desktop_mode.width - w)/2;
                 }
-	    }
+                if (desktop_mode.height>h)
+                {
+                    pos.y = (desktop_mode.height - h)/2;
+                }
+
+                pos.x+=desktopextent.ul.x;
+                pos.y+=desktopextent.ul.y;
+                PtSetArg(&args[nargs++], Pt_ARG_POS, &pos, 0);
+            }
+            else
+            {
+                if (windowpos)
+                {
+                    if (sscanf(windowpos, "%d,%d", &x, &y) == 2)
+                    {
+                        if ((x<desktop_mode.width) && (y<desktop_mode.height))
+                        {
+                            PhWindowQueryVisible(Ph_QUERY_CONSOLE, 0, 0, &desktopextent);
+                            pos.x=x+desktopextent.ul.x;
+                            pos.y=y+desktopextent.ul.y;
+                        }
+                        PtSetArg(&args[nargs++], Pt_ARG_POS, &pos, 0);
+                    }
+                }
+            }
         }
 
+        /* if window is maximized render it as maximized */
+        if (currently_maximized)
+        {
+           PtSetArg(&args[nargs++], Pt_ARG_WINDOW_STATE, Pt_TRUE, Ph_WM_STATE_ISMAX);
+        }
+        else
+        {
+           PtSetArg(&args[nargs++], Pt_ARG_WINDOW_STATE, Pt_FALSE, Ph_WM_STATE_ISMAX);
+        }
 
-        PtSetArg(&args[nargs++], Pt_ARG_FILL_COLOR, Pg_BLACK, 0);
-        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_STATE, Pt_FALSE, Ph_WM_STATE_ISFRONT | Ph_WM_STATE_ISMAX | Ph_WM_STATE_ISALTKEY);
+        /* do not grab the keyboard by default */
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_STATE, Pt_FALSE, Ph_WM_STATE_ISALTKEY);
+
+        /* bring the focus to the window */
+        PtSetArg(&args[nargs++], Pt_ARG_WINDOW_STATE, Pt_TRUE, Ph_WM_STATE_ISFOCUS);
+
+        /* allow to catch hide event */
         PtSetArg(&args[nargs++], Pt_ARG_WINDOW_MANAGED_FLAGS, Pt_TRUE, Ph_WM_HIDE);
         PtSetArg(&args[nargs++], Pt_ARG_WINDOW_NOTIFY_FLAGS, Pt_TRUE, Ph_WM_HIDE);
-        PtSetArg(&args[nargs++], Pt_ARG_RESIZE_FLAGS, Pt_FALSE, Pt_RESIZE_XY_AS_REQUIRED);
     }
 
     PtSetResources(window, nargs, args);
     PtRealizeWidget(window);
     PtWindowToFront(window);
 
+#if 0 /* FIXME */
+    PtGetResource(window, Pt_ARG_POS, &olddim, 0);
+    fprintf(stderr, "POSITION: %d, %d\n", olddim->w, olddim->h);
+#endif
+
     return 0;
 }
 
@@ -305,9 +361,9 @@ static const struct ColourMasks* ph_GetColourMasks(int bpp)
     return NULL;
 }
 
-static int ph_VideoInit(_THIS, SDL_PixelFormat *vformat)
+static int ph_VideoInit(_THIS, SDL_PixelFormat* vformat)
 {
-    PgHWCaps_t my_hwcaps;
+    PgHWCaps_t hwcaps;
     int i;
 
     window=NULL;
@@ -346,14 +402,14 @@ static int ph_VideoInit(_THIS, SDL_PixelFormat *vformat)
         return -1;
     }
 
-    if (PgGetGraphicsHWCaps(&my_hwcaps) < 0)
+    if (PgGetGraphicsHWCaps(&hwcaps) < 0)
     {
         SDL_SetError("ph_VideoInit(): GetGraphicsHWCaps function failed !\n");
         this->FreeWMCursor(this, SDL_BlankCursor);
         return -1;
     }
 
-    if (PgGetVideoModeInfo(my_hwcaps.current_video_mode, &desktop_mode) < 0)
+    if (PgGetVideoModeInfo(hwcaps.current_video_mode, &desktop_mode) < 0)
     {
         SDL_SetError("ph_VideoInit(): PgGetVideoModeInfo function failed !\n");
         this->FreeWMCursor(this, SDL_BlankCursor);
@@ -382,6 +438,7 @@ static int ph_VideoInit(_THIS, SDL_PixelFormat *vformat)
          
     currently_fullscreen = 0;
     currently_hided = 0;
+    currently_maximized = 0;
     current_overlay = NULL;
 
     OCImage.direct_context = NULL;
@@ -391,16 +448,22 @@ static int ph_VideoInit(_THIS, SDL_PixelFormat *vformat)
     OCImage.CurrentFrameData = NULL;
     OCImage.FrameData0 = NULL;
     OCImage.FrameData1 = NULL;
-
+    videomode_emulatemode = 0;
     
+    this->info.video_mem=hwcaps.currently_available_video_ram/1024;
     this->info.wm_available = 1;
+    this->info.hw_available = 1;
+    this->info.blit_fill = 1;
+    this->info.blit_hw = 1;
+    this->info.blit_hw_A = 0;
+    this->info.blit_hw_CC = 1;
     
     return 0;
 }
 
-static SDL_Surface *ph_SetVideoMode(_THIS, SDL_Surface *current,
-                int width, int height, int bpp, Uint32 flags)
+static SDL_Surface* ph_SetVideoMode(_THIS, SDL_Surface *current, int width, int height, int bpp, Uint32 flags)
 {
+    PgHWCaps_t hwcaps;
     const struct ColourMasks* mask;
 
     /* Lock the event thread, in multi-threading environments */
@@ -429,13 +492,11 @@ static SDL_Surface *ph_SetVideoMode(_THIS, SDL_Surface *current,
         return NULL;
     }
 
-#ifdef HAVE_OPENGL
     if ((current->flags & SDL_OPENGL)==SDL_OPENGL)
     {
-#else
-    if ((current->flags & SDL_OPENGL)==SDL_OPENGL) /* if no built-in OpenGL support */
-    {
-        SDL_SetError("ph_SetVideoMode(): no OpenGL support, try to recompile library.\n");
+#if !defined(HAVE_OPENGL)
+        /* if no built-in OpenGL support */
+        SDL_SetError("ph_SetVideoMode(): no OpenGL support, you need to recompile SDL.\n");
         current->flags &= ~SDL_OPENGL;
         return NULL;
 #endif /* HAVE_OPENGL */
@@ -455,13 +516,12 @@ static SDL_Surface *ph_SetVideoMode(_THIS, SDL_Surface *current,
         }
         else
         {
-            /* remove this if we'll support non-fullscreen sw/hw+doublebuf */
+            /* remove this if we'll have support for the non-fullscreen sw/hw+doublebuf one day */
             current->flags &= ~SDL_DOUBLEBUF;
 
             /* Use offscreen memory if SDL_HWSURFACE flag is set */
             if ((current->flags & SDL_HWSURFACE) == SDL_HWSURFACE)
             {
-
                 if (desktopbpp!=bpp)
                 {
                    current->flags &= ~SDL_HWSURFACE;
@@ -511,9 +571,18 @@ static SDL_Surface *ph_SetVideoMode(_THIS, SDL_Surface *current,
        PgFlush();
     }
 
+    visualbpp=bpp;
+
+    if (PgGetGraphicsHWCaps(&hwcaps) < 0)
+    {
+        SDL_SetError("ph_SetVideoMode(): GetGraphicsHWCaps function failed !\n");
+        return NULL;
+    }
+    this->info.video_mem=hwcaps.currently_available_video_ram/1024;
+
     SDL_Unlock_EventThread();
 
-    /* We're done! */
+    /* We've done! */
     return (current);
 }
 
