@@ -1,6 +1,6 @@
 /*
 	SDL - Simple DirectMedia Layer
-	Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 	Sam Lantinga
-	slouken@devolution.com
+	slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_gsevents.c,v 1.2 2001/07/31 06:08:11 slouken Exp $";
+ "@(#) $Id: SDL_gsevents.c,v 1.5 2004/01/04 16:49:26 slouken Exp $";
 #endif
 
 /* Handle the event stream, converting console events into SDL events */
