@@ -1,3 +1,27 @@
+/*
+    SDL - Simple DirectMedia Layer
+    Copyright (C) 1997-2003  Sam Lantinga
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    Sam Lantinga
+    slouken@libsdl.org
+*/
+
+#include "SDL_QuartzVideo.h"
+#include "SDL_QuartzWindow.h"
 
 /*
     This function makes the *SDL region* of the window 100% opaque. 
@@ -25,18 +49,6 @@ static void QZ_SetPortAlphaOpaque () {
     }
 }
 
-/* Subclass of NSWindow to fix genie effect and support resize events  */
-@interface SDL_QuartzWindow : NSWindow
-{}
-- (void)miniaturize:(id)sender;
-- (void)display;
-- (void)setFrame:(NSRect)frameRect display:(BOOL)flag;
-- (void)appDidHide:(NSNotification*)note;
-- (void)appWillUnhide:(NSNotification*)note;
-- (void)appDidUnhide:(NSNotification*)note;
-- (id)initWithContentRect:(NSRect)contentRect styleMask:(unsigned int)styleMask backing:(NSBackingStoreType)backingType defer:(BOOL)flag;
-@end
-
 @implementation SDL_QuartzWindow
 
 /* we override these methods to fix the miniaturize animation/dock icon bug */
@@ -186,24 +198,33 @@ static void QZ_SetPortAlphaOpaque () {
 
 @end
 
-/* Delegate for our NSWindow to send SDLQuit() on close */
-@interface SDL_QuartzWindowDelegate : NSObject
-{}
-- (BOOL)windowShouldClose:(id)sender;
-@end
-
 @implementation SDL_QuartzWindowDelegate
 - (BOOL)windowShouldClose:(id)sender
 {
     SDL_PrivateQuit();
     return NO;
 }
-@end
 
-/* Subclass of NSQuickDrawView for the window's subview */
-@interface SDL_QuartzWindowView : NSQuickDrawView
-{}
-@end
+- (void)windowDidBecomeKey:(NSNotification *)aNotification
+{
+    SDL_PrivateAppActive (1, SDL_APPINPUTFOCUS);
+}
+
+- (void)windowDidResignKey:(NSNotification *)aNotification
+{
+    SDL_PrivateAppActive (0, SDL_APPINPUTFOCUS);
+}
+
+- (void)windowDidBecomeMain:(NSNotification *)aNotification
+{
+    SDL_VideoDevice *this = (SDL_VideoDevice*)current_video;
+    if (this && QZ_IsMouseInWindow (this))
+        SDL_PrivateAppActive (1, SDL_APPMOUSEFOCUS);
+}
+
+- (void)windowDidResignMain:(NSNotification *)aNotification
+{
+    SDL_PrivateAppActive (0, SDL_APPMOUSEFOCUS);
+}
 
-@implementation SDL_QuartzWindowView
 @end
